package vn.teko.loyalty.loyalty_flutter

import vn.teko.loyalty.core.MemberInfoRequestType
import vn.teko.loyalty.core.model.api.query.TransactionHistoryQuery
import vn.teko.loyalty.core.model.api.request.RegisterMemberRequest
import vn.teko.loyalty.core.model.configuration.LoyaltyConfiguration
import vn.teko.loyalty.core.model.member.EncryptedMemCode
import vn.teko.loyalty.core.model.member.MemberInfo
import vn.teko.loyalty.core.model.member.RegisterMemberResult
import vn.teko.loyalty.core.model.network.NetworkConfigResult
import vn.teko.loyalty.core.model.transaction.Transaction
import vn.teko.loyalty.core.model.transaction.TransactionHistoryResult

object ModelConverter {
    fun convertLoyaltyConfiguration(config: LoyaltyConfiguration): LoyaltyFlutter.FlutterLoyaltyConfigurationResult {
        val result = LoyaltyFlutter.FlutterLoyaltyConfigurationResult().apply { isSuccess = true }
        val flutterConfig = LoyaltyFlutter.FlutterLoyaltyConfiguration().apply {
            url = config.url
            bffUrl = config.bffUrl
            merchantCode = config.merchantCode
            clientId = config.clientId
            iamAudience = config.iamAudience
            loggingEnabled = config.loggingEnabled
            linkMembershipUrl = config.linkMembershipUrl
            registerMemberUrl = config.registerMemberUrl
            allowLinkMembership = config.allowLinkMembership
            allowRegisterMembership = config.allowRegisterMembership
            membershipCodeFormat = config.membershipCodeFormat
        }
        result.loyaltyConfiguration = flutterConfig

        return result
    }

    fun convertLoyaltyRequest(request: LoyaltyFlutter.FlutterRegisterMemeberRequest): RegisterMemberRequest {
        val gender = when (request.gender) {
            LoyaltyFlutter.FlutterGender.female -> RegisterMemberRequest.Gender.FEMALE
            LoyaltyFlutter.FlutterGender.male -> RegisterMemberRequest.Gender.MALE
            LoyaltyFlutter.FlutterGender.none -> RegisterMemberRequest.Gender.NONE
            else -> RegisterMemberRequest.Gender.NONE
        }

        return RegisterMemberRequest(
            phone = request.phone ?: "",
            name = request.name ?: "",
            address = request.address ?: "",
            dateOfBirth = request.dateOfBirth ?: "",
            email = request.email ?: "",
            gender = gender,
            idCardNumber = request.idCardNumber ?: "",
            idCitizenNumber = request.idCitizenNumber ?: "",
            passportNumber = request.passportNumber ?: "",
        )
    }

    fun convertRegisterMemberResult(registerMemberResult: RegisterMemberResult): LoyaltyFlutter.FlutterRegisterMemberResult {
        return LoyaltyFlutter.FlutterRegisterMemberResult().apply {
            isSuccess = true
            registerMember = LoyaltyFlutter.FlutterRegisterMember().also { registerMember ->
                registerMember.memberId = registerMemberResult.memberId
            }
        }
    }

    fun convertMembershipQrResult(encryptedMemCode: EncryptedMemCode): LoyaltyFlutter.FlutterMembershipQrResult {
        val membershipQr = LoyaltyFlutter.FlutterMembershipQr().apply {
            code = encryptedMemCode.code
            issueAt = encryptedMemCode.issueAt
            expireAt = encryptedMemCode.expireAt
        }

        return LoyaltyFlutter.FlutterMembershipQrResult().apply {
            this.isSuccess = true
            this.membershipQr = membershipQr
        }
    }

    fun convertMemberInfoRequest(request: LoyaltyFlutter.FlutterMemberInfoRequest): MemberInfoRequestType {
        if (request.phone.isNullOrBlank() && request.memberQrCode.isNullOrBlank()) {
            return MemberInfoRequestType.CurrentUser
        }

        if (!request.phone.isNullOrBlank()) {
            return MemberInfoRequestType.Phone(request.phone!!)
        }

        return MemberInfoRequestType.MemberQrCode(request.memberQrCode!!)
    }

    fun convertMemberInfoResult(memberInfo: MemberInfo): LoyaltyFlutter.FlutterMemberInfoResult {
        val flutterResult = LoyaltyFlutter.FlutterMemberInfoResult().apply { isSuccess = true }

        val type = when (memberInfo.defaultCardType) {
            MemberInfo.CardType.MEMBER_CARD -> LoyaltyFlutter.FlutterCardType.memberCard
            else -> LoyaltyFlutter.FlutterCardType.qr
        }
        val flutterMemberInfo = LoyaltyFlutter.FlutterMemberInfo().apply {
            memberQrCode = memberInfo.memberQrCode
            memberId = memberInfo.memberId
            phone = memberInfo.phone
            memberCardId = memberInfo.memberCardId
            name = memberInfo.name
            isActive = memberInfo.isActive
            point = memberInfo.point
            tierPoint = memberInfo.tierPoint
            keepTierUntil = memberInfo.keepTierUntil
            accumulationFrom = memberInfo.accumulationFrom
            accumulationTo = memberInfo.accumulationTo
            currentTierCode = memberInfo.currentTierCode
            currentTierName = memberInfo.currentTierName
            currentTierMinPoint = memberInfo.currentTierMinPoint?.toLong()
            nextTierName = memberInfo.nextTierName
            nextTierMinPoint = memberInfo.nextTierMinPoint?.toLong()
            defaultCardType = type
        }

        flutterResult.flutterMemberInfo = flutterMemberInfo
        return flutterResult
    }

    fun convertTransactionHistoryRequest(request: LoyaltyFlutter.FlutterTransactionHistoryRequest): TransactionHistoryQuery {
        val kotlinTransactionType = Transaction.TransactionType.values();
        val flutterTxnTypes = request.txnTypes?.map { txt ->
            kotlinTransactionType.firstOrNull { it.value == txt }
                ?: Transaction.TransactionType.GRANT
        }

        val status = when (request.status) {
            LoyaltyFlutter.FlutterStatus.init -> Transaction.Status.INIT
            LoyaltyFlutter.FlutterStatus.initFailure -> Transaction.Status.INIT_FAILURE
            LoyaltyFlutter.FlutterStatus.noAccouning -> Transaction.Status.NO_ACCOUNTING
            LoyaltyFlutter.FlutterStatus.accountingSuccess -> Transaction.Status.ACCOUNTING_SUCCESS
            LoyaltyFlutter.FlutterStatus.accountingFailure -> Transaction.Status.ACCOUNTING_FAILURE
            LoyaltyFlutter.FlutterStatus.accouting -> Transaction.Status.ACCOUNTING
            else -> Transaction.Status.INIT
        }


        return TransactionHistoryQuery().apply {
            this.merchantCode = request.merchantCode
            this.fromTime = request.fromTime
            this.toTime = request.toTime
            this.txnTypes = flutterTxnTypes
            this.status = status
            this.page = request.page?.toInt()
            this.pageSize = request.pageSize?.toInt()
        }
    }

    fun convertTransactionHistoryResult(transactionHistoryResult: TransactionHistoryResult): LoyaltyFlutter.FlutterTransactionHistoryResult {
        val flutterResult =
            LoyaltyFlutter.FlutterTransactionHistoryResult().apply { isSuccess = true }

        val transactions = transactionHistoryResult.transactions.map { transaction ->
            val txtType = when (transaction.txnType) {
                Transaction.TransactionType.GRANT -> LoyaltyFlutter.FlutterTransactionType.grant
                Transaction.TransactionType.SPEND -> LoyaltyFlutter.FlutterTransactionType.spend
                Transaction.TransactionType.RETURN_GRANT -> LoyaltyFlutter.FlutterTransactionType.returnGrant
                Transaction.TransactionType.RETURN_SPEND -> LoyaltyFlutter.FlutterTransactionType.returnSpend
                Transaction.TransactionType.INITIATE -> LoyaltyFlutter.FlutterTransactionType.initiate
                Transaction.TransactionType.INITIATE_PROMOTION -> LoyaltyFlutter.FlutterTransactionType.initiatePromotion
                Transaction.TransactionType.ISSUE -> LoyaltyFlutter.FlutterTransactionType.issue
                else -> LoyaltyFlutter.FlutterTransactionType.grant
            }

            val accountingType = when (transaction.accountingType) {
                Transaction.AccountingType.CREDIT -> LoyaltyFlutter.FlutterAccountingType.credit
                else -> LoyaltyFlutter.FlutterAccountingType.debit
            }

            val status = when (transaction.status) {
                Transaction.Status.INIT -> LoyaltyFlutter.FlutterStatus.init
                Transaction.Status.INIT_FAILURE -> LoyaltyFlutter.FlutterStatus.initFailure
                Transaction.Status.NO_ACCOUNTING -> LoyaltyFlutter.FlutterStatus.noAccouning
                Transaction.Status.ACCOUNTING_SUCCESS -> LoyaltyFlutter.FlutterStatus.accountingSuccess
                Transaction.Status.ACCOUNTING_FAILURE -> LoyaltyFlutter.FlutterStatus.accountingFailure
                Transaction.Status.ACCOUNTING -> LoyaltyFlutter.FlutterStatus.accouting
                else -> LoyaltyFlutter.FlutterStatus.init
            }

            LoyaltyFlutter.FlutterTransaction().apply {
                this.id = transaction.id
                this.refId = transaction.refId
                this.orderCode = transaction.orderCode
                this.txnType = txnType
                this.txnTypeText = transaction.txnTypeText
                this.txnDesc = transaction.txnDesc
                this.accountingType = accountingType
                this.status = status
                this.point = transaction.point.toLong()
                this.createdAt = transaction.createdAt
                this.createdBy = transaction.createdBy
                this.networkId = transaction.networkId
                this.networkName = transaction.networkName
                this.merchantCode = transaction.merchantCode
                this.merchantName = transaction.merchantName
                this.terminalCode = transaction.terminalCode
                this.terminalName = transaction.terminalName
                this.terminalAddress = transaction.terminalAddress
            }
        }
        val pageData = LoyaltyFlutter.FlutterPageData().apply {
            this.currentPage = transactionHistoryResult.pageData.currentPage.toLong()
            this.totalPage = transactionHistoryResult.pageData.totalPage.toLong()
            this.pageSize = transactionHistoryResult.pageData.pageSize.toLong()
            this.totalItems = transactionHistoryResult.pageData.totalItems.toLong()
        }

        flutterResult.transactionHistory = LoyaltyFlutter.FlutterTransactionHistory().apply {
            this.transactions = transactions
            this.pageData = pageData
        }

        return flutterResult
    }

    fun convertNetworkConfig(networkConfigResult: NetworkConfigResult): LoyaltyFlutter.FlutterNetworkConfigResult {
        val flutterResult = LoyaltyFlutter.FlutterNetworkConfigResult().apply { isSuccess = true }

        val networkInfo = LoyaltyFlutter.FlutterNetwork().also {
            it.id = networkConfigResult.networkInfo.id
            it.name = networkConfigResult.networkInfo.name
        }

        val networkConfig = LoyaltyFlutter.FlutterNetworkConfig().also {
            it.exchangeRate = networkConfigResult.networkConfig.exchangeRate.toLong()
            it.currencyImg = networkConfigResult.networkConfig.currencyImg
            it.currencyName = networkConfigResult.networkConfig.currencyName
            it.cardDefaultImg = networkConfigResult.networkConfig.cardDefaultImg
            it.avatarDefaultImg = networkConfigResult.networkConfig.avatarDefaultImg
            it.allowToGetMemberByPhone = networkConfigResult.networkConfig.allowToGetMemberByPhone
            it.faqWebUrl = networkConfigResult.networkConfig.faqWebUrl
            it.benefitWebUrl = networkConfigResult.networkConfig.benefitWebUrl
            it.brandColor = networkConfigResult.networkConfig.brandColor
            it.allowMerchantIssueMemberCard =
                networkConfigResult.networkConfig.allowMerchantIssueMemberCard
        }

        val networkTiers = networkConfigResult.networkTiers?.map { tier ->
            LoyaltyFlutter.FlutterTier().also {
                it.code = tier.code
                it.name = tier.name
                it.fromPoint = tier.fromPoint.toLong()
                it.cardTieredImg = tier.cardTieredImg
                it.tierIconImg = tier.tierIconImg
            }
        }

        flutterResult.networkConfig = LoyaltyFlutter.FlutterNetworkData().also {
            it.networkInfo = networkInfo
            it.networkTiers = networkTiers
            it.networkConfig = networkConfig
        }

        return  flutterResult
    }
}